#!/bin/bash

# Build config for the build script, build.sh. Look there for more info.

APP_NAME=messagingmenu
CHROME_PROVIDERS="content locale res"
CLEAN_UP=1
ROOT_FILES=
ROOT_DIRS="defaults"
BEFORE_BUILD=
BEFORE_PACK=
AFTER_BUILD=
